@@ -21,6 +21,26 @@ static PyObject *PyShmInfo = NULL;
 /* memory page size. this will be the default max size for msgrcv */
 static int pagesize;
 
+#ifdef __GLIBC__
+	#define key __key
+	#define seq __seq
+	#define msg_cbytes __msg_cbytes
+
+	#ifdef _GNU_SOURCE
+		#define USE_SEMTIMEDOP
+
+		#ifdef IPC_INFO
+			#define USE_IPC_INFO
+		#endif
+		#ifdef SEM_INFO
+			#define USE_SEM_INFO
+		#endif
+		#ifdef SHM_INFO
+			#define USE_SHM_INFO
+		#endif
+	#endif
+#endif
+
 
 static PyObject *
 pythonify_ipcperm(struct ipc_perm *perm) {
@@ -28,7 +48,7 @@ pythonify_ipcperm(struct ipc_perm *perm) {
 
     if (NULL == (args = PyTuple_New(7))) return NULL;
 
-    if (NULL == (obj = PyInt_FromLong((long)perm->__key & 0xffffffff)))
+    if (NULL == (obj = PyInt_FromLong((long)perm->key & 0xffffffff)))
         goto fail;
     PyTuple_SET_ITEM(args, 0, obj);
 
@@ -52,7 +72,7 @@ pythonify_ipcperm(struct ipc_perm *perm) {
         goto fail;
     PyTuple_SET_ITEM(args, 5, obj);
 
-    if (NULL == (obj = PyInt_FromLong((long)perm->__seq)))
+    if (NULL == (obj = PyInt_FromLong((long)perm->seq)))
         goto fail;
     PyTuple_SET_ITEM(args, 6, obj);
 
@@ -90,7 +110,7 @@ pythonify_mqds(struct msqid_ds *mqds) {
         goto fail;
     PyTuple_SET_ITEM(args, 3, obj);
 
-    if (NULL == (obj = PyLong_FromUnsignedLong(mqds->__msg_cbytes)))
+    if (NULL == (obj = PyLong_FromUnsignedLong(mqds->msg_cbytes)))
         goto fail;
     PyTuple_SET_ITEM(args, 4, obj);
 
@@ -217,6 +237,8 @@ pythonify_semarray(unsigned short *semvals, unsigned long count) {
     return result;
 }
 
+#ifdef USE_IPC_INFO
+
 static PyObject *
 pythonify_msginfo(struct msginfo *info) {
     PyObject *obj, *args, *result = NULL;
@@ -263,6 +285,9 @@ done:
     return result;
 }
 
+#endif
+#ifdef USE_SEM_INFO
+
 static PyObject *
 pythonify_seminfo(struct seminfo *info) {
     PyObject *obj, *args, *result = NULL;
@@ -317,6 +342,9 @@ done:
     return result;
 }
 
+#endif
+#ifdef USE_SHM_INFO
+
 static PyObject *
 pythonify_shminfo(struct shm_info *info) {
     PyObject *args, *obj, *result = NULL;
@@ -347,6 +375,8 @@ done:
     return result;
 }
 
+#endif
+
 int
 pytolong(PyObject *obj, long *target) {
     if (PyInt_Check(obj)) {
@@ -456,7 +486,7 @@ unpythonify_semops(PyObject *pyops, semops *ops) {
         return -1;
 
     i = 0;
-    while (tuple = PyIter_Next(iter)) {
+    while ((tuple = PyIter_Next(iter))) {
         if (i == ops->count) {
             PyErr_SetString(PyExc_RuntimeError, "overrun");
             return -1;
@@ -519,7 +549,7 @@ unpythonify_semarray(PyObject *pyarray, unsigned short *array) {
     if (NULL == (iter = PyObject_GetIter(pyarray)))
         return -1;
 
-    while (obj = PyIter_Next(iter)) {
+    while ((obj = PyIter_Next(iter))) {
         if (pytolong(obj, &l) < 0) {
             Py_DECREF(obj);
             Py_DECREF(iter);
@@ -546,7 +576,9 @@ python_msgctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     PyObject *setinfo = NULL;
     struct msqid_ds mqds;
     struct msqid_ds *mqdsp = &mqds;
+#ifdef USE_IPC_INFO
     struct msginfo mi;
+#endif
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ii|O", msgctl_kwargs,
                 &mqid, &cmd, &setinfo))
@@ -554,7 +586,7 @@ python_msgctl(PyObject *self, PyObject *args, PyObject *kwargs) {
 
     if (cmd == IPC_SET && unpythonify_msgsetinfo(setinfo, mqdsp))
         return NULL;
-#ifdef IPC_INFO
+#ifdef USE_IPC_INFO
     if (cmd == IPC_INFO || cmd == MSG_INFO)
         mqdsp = (struct msqid_ds *)&mi;
 #endif
@@ -571,7 +603,7 @@ python_msgctl(PyObject *self, PyObject *args, PyObject *kwargs) {
             )
         return pythonify_mqds(mqdsp);
 
-#ifdef IPC_INFO
+#ifdef USE_IPC_INFO
     if (cmd == IPC_INFO || cmd == MSG_INFO)
         return pythonify_msginfo(&mi);
 #endif
@@ -735,7 +767,9 @@ union semun {
     int val;
     struct semid_ds *buf;
     unsigned short *array;
+#ifdef USE_SEM_INFO
     struct seminfo *__buf;
+#endif
 };
 
 static char *semctl_kwargs[] = {"id", "cmd", "semnum", "arg", "sizehint", NULL};
@@ -746,7 +780,9 @@ python_semctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     long nsems = -1;
     PyObject *info = NULL;
     struct semid_ds sds;
+#ifdef USE_SEM_INFO
     struct seminfo si;
+#endif
     union semun sun;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ii|iOl", semctl_kwargs,
@@ -759,7 +795,7 @@ python_semctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     if (cmd == IPC_SET || cmd == IPC_STAT || cmd == SEM_STAT)
         sun.buf = &sds;
 
-#ifdef IPC_INFO
+#ifdef USE_SEM_INFO
     if (cmd == IPC_INFO || cmd == SEM_INFO)
         sun.__buf = &si;
 #endif
@@ -811,7 +847,7 @@ python_semctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     if (cmd == IPC_STAT || cmd == SEM_STAT)
         return pythonify_sds(&sds);
 
-#ifdef IPC_INFO
+#ifdef USE_SEM_INFO
     if (cmd == IPC_INFO || cmd == SEM_INFO)
         return pythonify_seminfo(&si);
 #endif
@@ -844,12 +880,17 @@ python_semop(PyObject *module, PyObject *args, PyObject *kwargs) {
     int semid;
     PyObject *pyops;
     long len;
+#ifdef USE_SEMTIMEDOP
     double dtimeout = -1;
     struct timespec timeout;
     struct timespec *timeoutp = &timeout;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "iO|d", semop_kwargs,
                 &semid, &pyops, &dtimeout))
+#else
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "iO", semop_kwargs,
+                &semid, &pyops))
+#endif
         return NULL;
 
     if ((len = PyObject_Length(pyops)) < 0) {
@@ -861,6 +902,7 @@ python_semop(PyObject *module, PyObject *args, PyObject *kwargs) {
     semops ops = {(unsigned int)len, &bufs[0]};
     if (unpythonify_semops(pyops, &ops) < 0) return NULL;
 
+#ifdef USE_SEMTIMEDOP
     if (dtimeout > 0) {
         if (timespec_ify(dtimeout, timeoutp)) {
             PyErr_SetString(PyExc_ValueError, "issue with timeout float");
@@ -870,6 +912,9 @@ python_semop(PyObject *module, PyObject *args, PyObject *kwargs) {
         timeoutp = NULL;
 
     if (semtimedop(semid, ops.ops, (unsigned int)len, timeoutp) < 0) {
+#else
+    if (semop(semid, ops.ops, (unsigned int)len) < 0) {
+#endif
         PyErr_SetFromErrno(PyExc_OSError);
         return NULL;
     }
@@ -917,7 +962,9 @@ python_shmctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     int shmid, cmd;
     PyObject *pyinfo = NULL;
     struct shmid_ds sds;
+#ifdef USE_SHM_INFO
     struct shm_info si;
+#endif
     struct shmid_ds *sdsp = &sds;
     int result;
     
@@ -928,7 +975,7 @@ python_shmctl(PyObject *self, PyObject *args, PyObject *kwargs) {
     if (cmd == IPC_SET && unpythonify_shmsetinfo(pyinfo, sdsp))
         return NULL;
 
-#ifdef SHM_INFO
+#ifdef USE_SHM_INFO
     if (cmd == SHM_INFO)
         sdsp = (struct shmid_ds *)&si;
 #endif
@@ -945,7 +992,7 @@ python_shmctl(PyObject *self, PyObject *args, PyObject *kwargs) {
             )
         return pythonify_shmds(sdsp);
 
-#ifdef SHM_INFO
+#ifdef USE_SHM_INFO
     if (cmd == SHM_INFO)
         return pythonify_shminfo(&si);
 #endif
@@ -1489,7 +1536,9 @@ initsysv_ipc(void) {
     PyModule_AddIntConstant(module, "IPC_SET", IPC_SET);
     PyModule_AddIntConstant(module, "IPC_RMID", IPC_RMID);
 
+#ifdef MSG_EXCEPT
     PyModule_AddIntConstant(module, "MSG_EXCEPT", MSG_EXCEPT);
+#endif
     PyModule_AddIntConstant(module, "MSG_NOERROR", MSG_NOERROR);
 
     PyModule_AddIntConstant(module, "SEM_UNDO", SEM_UNDO);
@@ -1502,14 +1551,16 @@ initsysv_ipc(void) {
     PyModule_AddIntConstant(module, "SETVAL", SETVAL);
 
     PyModule_AddIntConstant(module, "SHM_RDONLY", SHM_RDONLY);
+#ifdef SHM_DEST
     PyModule_AddIntConstant(module, "SHM_DEST", SHM_DEST);
-#ifdef SHM_INFO
+#endif
+#ifdef USE_SHM_INFO
     PyModule_AddIntConstant(module, "SHM_INFO", SHM_INFO);
 #endif
 #ifdef SHM_STAT
     PyModule_AddIntConstant(module, "SHM_STAT", SHM_STAT);
 #endif
-#ifdef IPC_INFO
+#ifdef USE_IPC_INFO
     PyModule_AddIntConstant(module, "IPC_INFO", IPC_INFO);
 #endif
 #ifdef MSG_INFO
@@ -1518,7 +1569,7 @@ initsysv_ipc(void) {
 #ifdef MSG_STAT
     PyModule_AddIntConstant(module, "MSG_STAT", MSG_STAT);
 #endif
-#ifdef SEM_INFO
+#ifdef USE_SEM_INFO
     PyModule_AddIntConstant(module, "SEM_INFO", SEM_INFO);
 #endif
 #ifdef SEM_STAT
